@@ -5,8 +5,9 @@
 #' @param males A vector with male numbers
 #' @param females A vector with female numbers
 #' @param N A vector with total numbers
-#' @param temperatures The constant incubation temperatures or any covariate used to fit sex ratio
-#' @param df A dataframe with at least two columns named males, females or N and temperatures column
+#' @param temperatures The constant incubation temperatures used to fit sex ratio
+#' @param durations The duration of incubation or TSP used to fit sex ratio
+#' @param df A dataframe with at least two columns named males, females or N and temperatures, Incubation.temperature or durations column
 #' @param l The limit to define TRT (see Girondot, 1999)
 #' @param parameters.initial Initial values for P, S or K search as a vector, ex. c(P=29, S=-0.3)
 #' @param las.x las parameter for x axis
@@ -14,18 +15,23 @@
 #' @param lab.PT Label to describe pivotal temperature
 #' @param lab.TRT Label to describe transitional range of temperature
 #' @param males.freq Should the graph uses males frequency [TRUE] or females [FALSE]
-#' @param equation Could be "logistic", "Hill", "Richards" or "GSD"
+#' @param equation Could be "logistic", "Hill", "Richards", "Hulin", "Double-Richards" or "GSD"
 #' @param replicate Number of replicate to estimate SE of TRT
 #' @param range.CI The range of confidence interval for estimation, default=0.95
 #' @param col.TRT The color of TRT
 #' @param col.TRT.CI The color of CI of TRT based on range.CI
 #' @param col.PT.CI The color of CI of PT based on range.CI
-#' @param print Do the results must be printed at screen? TRUE or FALSE
+#' @param show.CI Do the CI for the curve should be shown
+#' @param limit.low.TRT.minimum Minimum lower limit for TRT
+#' @param limit.high.TRT.maximum Maximum higher limit for TRT
+#' @param print Do the results must be printed at screen? TRUE (default) or FALSE
+#' @param show.plot Do the plot must be shown? TRUE (default) or FALSE
 #' @param ... Graphical parameters for plot(), exemple xlab="", ylab="", main=""
 #' @description Estimate the parameters that best describe temperature-dependent sex determination
 #' @references Girondot, M. 1999. Statistical description of temperature-dependent sex determination using maximum likelihood. Evolutionary Ecology Research, 1, 479-486.
 #' @references Godfrey, M.H., Delmas, V., Girondot, M., 2003. Assessment of patterns of temperature-dependent sex determination using maximum likelihood model selection. Ecoscience 10, 265-272.
 #' @references Hulin, V., Delmas, V., Girondot, M., Godfrey, M.H., Guillon, J.-M., 2009. Temperature-dependent sex determination and global change: are some species at greater risk? Oecologia 160, 493-506.
+#' @references Girondot M., Submited. On the concept of embryological thermosensitive period for sex determination in reptiles.
 #' @examples
 #' \dontrun{
 #' CC_AtlanticSW <- subset(STSRE_TSD, RMU=="Atlantic, SW" & 
@@ -43,39 +49,90 @@
 #' gsd <- with (CC_AtlanticSW, tsd(males=Males, females=Females, 
 #'                                  temperatures=Incubation.temperature-Correction.factor, 
 #'                                  equation="GSD"))
-#' compare_AIC(Logistic_Model=tsdL, Hill_model=tsdH, Richards_model=tsdR, GSD_model=gsd)
+#' compare_AIC(Logistic_Model=tsdL, Hill_model=tsdH, Richards_model=tsdR, 
+#'                GSD_model=gsd)
+#' ##############
+#' eo <- subset(STSRE_TSD, Species=="Emys orbicularis", c("Males", "Females", 
+#'                                        "Incubation.temperature"))
+#'                                        
+#' eo_Hill <- with(eo, tsd(males=Males, females=Females, 
+#'                                        temperatures=Incubation.temperature,
+#'                                        equation="Hill"))
+#' eo_logistic <- tsd(eo)
+#' eo_Richards <- with(eo, tsd(males=Males, females=Females, 
+#'                                  temperatures=Incubation.temperature, 
+#'                                  equation="Richards"))
+#' ### The Hulin model is a modification of Richards (See Hulin et al. 2009)
+#' ### limit.low.TRT and limit.high.TRT must be setup for Hulin equation
+#' par <- eo_Richards$par
+#' names(par)[which(names(par)=="K")] <- "K2"
+#' par <- c(par, K1=0)
+#' eo_Hulin <- with(eo, tsd(males=Males, females=Females, 
+#'                                  parameters.initial=par, 
+#'                                  temperatures=Incubation.temperature, 
+#'                                  equation="Hulin", 
+#'                                  limit.low.TRT.minimum=25, 
+#'                                  limit.high.TRT.maximum=35))
+#' ### The Double-Richards model is a Richards model with K1 and K2 using the two values
+#' ### below and above P
+#' par <- eo_Richards$par
+#' names(par)[which(names(par)=="K")] <- "K2"
+#' par <- c(par, K1=as.numeric(par["K2"])-0.1)
+#' par["K1"] <- par["K1"]-0.1
+#' eo_Double_Richards <- with(eo, tsd(males=Males, females=Females,
+#'                                  parameters.initial=par,
+#'                                  temperatures=Incubation.temperature,
+#'                                  equation="Double-Richards"))
+#' ### Example with only points and mean curve
+#' eo_Double_Richards <- with(eo, tsd(males=Males, females=Females, 
+#'                                  parameters.initial=par, 
+#'                                  temperatures=Incubation.temperature, 
+#'                                  equation="Double-Richards", 
+#'                                  col.TRT="white", col.TRT.CI="white", col.PT.CI="white"))
+#' compare_AIC(Logistic=eo_logistic, Hill=eo_Hill, Richards=eo_Richards, 
+#'              Hulin=eo_Hulin, Double_Richards=eo_Double_Richards)
+#' ### Note the asymmetry of the Double-Richards model
+#' predict(eo_Double_Richards, 
+#'        temperatures=c(eo_Double_Richards$par["P"]-0.2, eo_Double_Richards$par["P"]+0.2))
+#' predict(eo_Double_Richards)
 #' }
 #' @export
 
 
-tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL, 
-	df=NULL, l=0.05, parameters.initial=c(P=NA, S=-0.5, K=0), males.freq=TRUE, 
+tsd <- function(df=NULL, males=NULL, females=NULL, N=NULL, temperatures=NULL, durations=NULL,
+	l=0.05, parameters.initial=c(P=NA, S=-0.5, K=0, K1=1, K2=0), males.freq=TRUE, 
 	las.x=1, las.y=1, lab.PT="Pivotal temperature", 
 	lab.TRT=paste0("Transitional range of temperatures l=",l*100,"%"), 
-	col.TRT="gray", col.TRT.CI=rgb(0.8, 0.8, 0.8, 0.5), col.PT.CI=rgb(0.8, 0.8, 0.8, 0.5),
-	equation="logistic", replicate=1000, range.CI=0.95, print=TRUE, ...) {
-  
-  # males <- c(10, 14, 7, 4, 3, 0, 0) ; females <- c(0, 1, 2, 4, 15, 10, 13)
-  # temperatures <- c(25, 26, 27, 28, 29, 30, 31)
-  # males <- NULL; females <- NULL; N <- NULL; temperatures <- NULL
+	col.TRT="gray", col.TRT.CI=rgb(0.8, 0.8, 0.8, 0.5), 
+  col.PT.CI=rgb(0.8, 0.8, 0.8, 0.5), show.CI=TRUE, 
+	equation="logistic", replicate=1000, range.CI=0.95, 
+  limit.low.TRT.minimum=5, limit.high.TRT.maximum=90, print=TRUE, show.plot=TRUE, ...) {
+
+# males=NULL; females=NULL; N=NULL; temperatures=NULL; durations=NULL; df <- NULL; l <- 0.05; parameters.initial=c(P=NA, S=-0.5, K=0, K1=1, K2=0); males.freq <- TRUE; las.x <- 1; las.y <- 1; lab.PT <- "Pivotal temperature"; lab.TRT <- "Transitional range of temperatures l=5%"; equation <- "logistic"; range.CI=0.95; replicate <- 1000; col.TRT="gray"; col.TRT.CI=rgb(0.8, 0.8, 0.8, 0.5); col.PT.CI=rgb(0.8, 0.8, 0.8, 0.5); limit.low.TRT.minimum=5; limit.high.TRT.maximum=90; print=TRUE; plot=TRUE
+# eo <- subset(STSRE_TSD, Species=="Emys orbicularis", c("Males", "Females", "Incubation.temperature"))
+# males=eo$Males; females=eo$Females; N <- males+females; temperatures=eo$Incubation.temperature; equation<- "Double-Richards" 
+
+  # males <- c(10, 14, 7, 4, 3, 0, 0) ; females <- c(0, 1, 2, 4, 15, 10, 13); temperatures <- c(25, 26, 27, 28, 29, 30, 31)
   
+  # males <- c(15L, 23L, 16L, 0L); females <- c(0L, 1L, 3L, 19L)
+  # temperatures <- c(70.4, 57.7, 53.9, 51.7)
+  # parameters.initial <- c(P=53, S=0.05)
+  # col.TRT="gray"; col.TRT.CI=rgb(0.8, 0.8, 0.8, 0.5); col.PT.CI=rgb(0.8, 0.8, 0.8, 0.5)
   # N <- NULL; males <- CC_AtlanticSW$Males;females <- CC_AtlanticSW$Females; temperatures<- CC_AtlanticSW$Incubation.temperature-CC_AtlanticSW$Correction.factor
   # equation <- "Richards"
-  # df <- NULL; l <- 0.05
-  # parameters.initial=c(P=NA, S=-0.5, K=0); males.freq <- TRUE
-  # las.x <- 1; las.y <- 1; lab.PT <- "Pivotal temperature"
-  # lab.TRT <- "Transitional range of temperatures l=5%"; equation <- "logistic"
-  # range.CI=0.95; replicate <- 1000; print=TRUE
+  # equation <- "Hill"
+  # equation <- "Double-Richards"
+  # equation <- "logistic"
   
   range.CI.qnorm <- qnorm(1-((1-range.CI)/2))
   
   if (!is.null(df)) {
-    if (class(df)!="data.frame") {
-      print("df parameter must be a data.frame")
+    if (class(df)!="data.frame" & class(df)!="matrix") {
+      print("df parameter must be a data.frame or a matrix")
       return(invisible())
     }
     
-    namesdf <- tolower(names(df))
+    namesdf <- tolower(colnames(df))
     males <- NULL
     females <- NULL
     N <- NULL
@@ -87,17 +144,28 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
     if (any(namesdf=="n")) N <- df[, which(namesdf=="n")]
     if (any(namesdf=="temperatures")) temperatures <- df[, which(namesdf=="temperatures")]
     if (any(namesdf=="temperature")) temperatures <- df[, which(namesdf=="temperature")]
+    if (any(namesdf=="incubation.temperature")) temperatures <- df[, which(namesdf=="incubation.temperature")]
+    if (any(namesdf=="durations")) durations <- df[, which(namesdf=="durations")]
+    if (any(namesdf=="duration")) durations <- df[, which(namesdf=="duration")]
   }
 
+  if (is.null(durations) & is.null(temperatures)) {
+    warning("Or temperatures or durations must be provided")
+    return()
+  }
+  
+  if (!is.null(durations) & !is.null(temperatures)) {
+    warning("Temperatures and durations cannot be both provided")
+    return()
+  }
+  
+  
   # si je retire des lignes, c'est décalé
 #  if (!is.null(males)) males <- na.omit(males)
 #  if (!is.null(females)) females <- na.omit(females)
 #  if (!is.null(N)) N <- na.omit(N)
   if (is.null(temperatures)) {
-    print("Temperatures must be provided")
-    return(invisible())
-#  } else {
-    #    temperatures <- na.omit(temperatures)
+    temperatures <- durations
   }
   
   cpt1 <- ifelse(is.null(males), 0, 1)+ifelse(is.null(females), 0, 1)+ifelse(is.null(N), 0, 1)
@@ -111,12 +179,12 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
   if (is.null(N)) N <- males+females
   
   if (length(temperatures)!=length(N)) {
-    print("A temperature must be provided for each experiment")
+    print("A temperature or duration must be provided for each experiment")
     return(invisible())    
   }
   
-  if (equation!="GSD" & equation!="logistic" & equation!="Hill" & equation!="Richards") {
-    print("Equations supported are GSD, logistic, Hill and Richards")
+  if (equation!="GSD" & equation!="Hulin" & equation!="logistic" & equation!="Hill" & equation!="Richards" & equation!="Double-Richards") {
+    print("Equations supported are GSD, logistic, Hill, Hulin, Double_Richards and Richards")
     return(invisible())    
   }
   
@@ -138,7 +206,10 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
     if (is.na(par["P"])) {
       par["P"] <- temperatures[which.min(abs((males/(males+females))-0.5))]
     }
-       if (equation=="logistic" | equation=="Hill") par <- par[which(names(par)!="K")]
+       if (equation!="Richards") par <- par[which(names(par)!="K")]
+       if (equation!="Hulin" & equation!="Double-Richards") par <- par[which(names(par)!="K1")]
+       if (equation!="Hulin" & equation!="Double-Richards") par <- par[which(names(par)!="K2")]
+
     repeat {
       result  <- optim(par, .tsd_fit, males=males, N=N, temperatures=temperatures, equation=equation, method="BFGS", hessian=TRUE, control = list(maxit=1000))
       # result  <- optim(par, embryogrowth:::.tsd_fit, males=males, N=N, temperatures=temperatures, equation=equation, method="BFGS", hessian=TRUE, control = list(maxit=1000))
@@ -156,11 +227,11 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
   
   if (inherits(inversemathessian, "try-error")) {
     res <- rep(NA, length(par))
+    names(res) <- names(par)
   } else {
     res <- abs(diag(inversemathessian))
   }
   
-  names(res) <- names(par)
   result$SE <- res
   result$AIC <- 2*result$value+2*length(par)
   }
@@ -169,6 +240,7 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
 	  l.l.TRT.c <- NULL
 	  l.h.TRT.c <- NULL
 	  TRT.c <- NULL
+ #   plist <- NULL
 	
 	for (j in 1:replicate) {
     
@@ -177,24 +249,70 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
 		  par.S <- rnorm(1, par["S"], ifelse(j==1, 0, res["S"]))
       if (sign(par.S)==sign(par["S"])) break
     }
-		if (equation=="Richards" & res["K"]!=0) {
-		repeat {
-		  par.K <- rnorm(1, par["K"], ifelse(j==1, 0, res["K"]))
-		  if (sign(par.K)==sign(par["K"])) break
+		if (equation=="Richards") {
+      if (res["K"]!=0) {
+		    repeat {
+		      par.K <- rnorm(1, par["K"], ifelse(j==1, 0, res["K"]))
+		      if (sign(par.K)==sign(par["K"])) break
+		    }
+      } else {par.K <- par["K"]}
 		}
-    } else {par.K <- par["K"]}
+ 
+		if (equation=="Double-Richards") {
+		    repeat {
+		      par.K1 <- rnorm(1, par["K1"], ifelse(j==1, 0, res["K1"]))
+		      if (sign(par.K1)==sign(par["K1"])) break
+		    }
+		    repeat {
+		      par.K2 <- rnorm(1, par["K2"], ifelse(j==1, 0, res["K2"]))
+		      if (sign(par.K2)==sign(par["K2"])) break
+		    }		    
+		}
+		
+		if (equation=="Hulin") {
+		  temperatures.se <- seq(from=limit.low.TRT.minimum, to=limit.high.TRT.maximum, length=20)
+      repeat {
+		  repeat {
+		    par.K1 <- rnorm(1, par["K1"], ifelse(j==1, 0, res["K1"]))
+		    if (sign(par.K1)==sign(par["K1"])) break
+		  }
+		  repeat {
+		    par.K2 <- rnorm(1, par["K2"], ifelse(j==1, 0, res["K2"]))
+		    if (sign(par.K2)==sign(par["K2"])) break
+		  }
+      p <- (1+(2^exp(par.K1*temperatures.se+par.K2)-1)*exp((1/par.S)*(par.P-temperatures.se)))^(-1/exp(par.K1*temperatures.se+par.K2))
+		  if (all(is.finite(p)) & max(p)>(1-l) & min(p)<l) break
+      }
+    }
     
-		limit.low.TRT <- 20
-		limit.high.TRT <- 40
+ #   plist <- c(plist, list(c(P=par.P, S=par.S, K1=par.K1, K2=par.K2)))
+		
+    # marche en températures mais par en IP - corrigé 9/9/2014
+		limit.low.TRT <- limit.low.TRT.minimum
+		limit.high.TRT <- limit.high.TRT.maximum
 		for (i in 1:5) {
 	  		temperatures.se <- seq(from=limit.low.TRT, to=limit.high.TRT, length=i*20)
 	  		if (equation=="logistic")	p <- 1/(1+exp((1/par.S)*(par.P-temperatures.se)))
  	  		if (equation=="Richards") p <- ifelse(par.K>3 & sign(par.P-temperatures.se)==sign(par.S), 
 	  		                                      0.5*exp((temperatures.se-par.P)/(par.S*exp(par.K))), 
 	  		                                      (1+(2^exp(par.K)-1)*exp((1/par.S)*(par.P-temperatures.se)))^(-1/exp(par.K)))
-	  		if (equation=="Hill") p <- 1/(1+exp((1/par.S)*(log(par.P)-log(temperatures.se))))
-			limit.low.TRT <- temperatures.se[tail(which(p>(1-l)), n=1)]
-			limit.high.TRT <- temperatures.se[which(p < l)[1]]
+	  		if (equation=="Double-Richards") p <- ifelse(temperatures.se<par.P, 
+	  		                                      ifelse(par.K1>3 & sign(par.P-temperatures.se)==sign(par.S), 
+	  		                                             0.5*exp((temperatures.se-par.P)/(par.S*exp(par.K1))), 
+	  		                                             (1+(2^exp(par.K1)-1)*exp((1/par.S)*(par.P-temperatures.se)))^(-1/exp(par.K1))) ,
+	  		                                     ifelse(par.K2>3 & sign(par.P-temperatures.se)==sign(par.S), 
+	  		                                             0.5*exp((temperatures.se-par.P)/(par.S*exp(par.K2))), 
+	  		                                             (1+(2^exp(par.K2)-1)*exp((1/par.S)*(par.P-temperatures.se)))^(-1/exp(par.K2)))
+                                              )
+        if (equation=="Hill") p <- 1/(1+exp((1/par.S)*(log(par.P)-log(temperatures.se))))
+	  		if (equation=="Hulin") p <- (1+(2^exp(par.K1*temperatures.se+par.K2)-1)*exp((1/par.S)*(par.P-temperatures.se)))^(-1/exp(par.K1*temperatures.se+par.K2))
+			if (sign(par.S)<0) {
+        limit.low.TRT <- temperatures.se[tail(which(p>(1-l)), n=1)]
+			  limit.high.TRT <- temperatures.se[which(p < l)[1]]
+			} else {
+			  limit.low.TRT <- temperatures.se[tail(which(p<l), n=1)]
+			  limit.high.TRT <- temperatures.se[which(p >(1-l))[1]]
+			}
 			limit.low.TRT <- ifelse(identical(limit.low.TRT, numeric(0)), NA, limit.low.TRT)
 			limit.high.TRT <- ifelse(identical(limit.high.TRT, numeric(0)), NA, limit.high.TRT)
       if (is.na(limit.low.TRT) | is.na(limit.high.TRT)) break
@@ -212,6 +330,7 @@ tsd <- function(males=NULL, females=NULL, N=NULL, temperatures=NULL,
 	limit.high.TRT <- result$TRT_limits[2]+range.CI.qnorm*result$SE_TRT_limits[2]
   }
 
+if (show.plot) {
    L <- list(...)
 
 	if (males.freq) {
@@ -302,6 +421,32 @@ if (equation!="GSD") {
 			0.5*exp((temperatures-par["P"])/(par["S"]*exp(par["K"]))), 
 			(1+(2^exp(par["K"])-1)*exp((1/par["S"])*(par["P"]-temperatures)))^(-1/exp(par["K"])))
 	}
+if (equation=="Double-Richards") {
+
+  p <- ifelse(x<par["P"],
+    
+    ifelse(par["K1"]>3 & sign(par["P"]-x)==sign(par["S"]), 
+              0.5*exp((x-par["P"])/(par["S"]*exp(par["K1"]))), 
+              (1+(2^exp(par["K1"])-1)*exp((1/par["S"])*(par["P"]-x)))^(-1/exp(par["K1"]))) ,
+    ifelse(par["K2"]>3 & sign(par["P"]-x)==sign(par["S"]), 
+           0.5*exp((x-par["P"])/(par["S"]*exp(par["K2"]))), 
+           (1+(2^exp(par["K2"])-1)*exp((1/par["S"])*(par["P"]-x)))^(-1/exp(par["K2"])))    
+  )
+
+  prop.theorique <- ifelse(temperatures<par["P"],
+                           ifelse(par["K1"]>3 & sign(par["P"]-temperatures)==sign(par["S"]), 
+                           0.5*exp((temperatures-par["P"])/(par["S"]*exp(par["K1"]))), 
+                           (1+(2^exp(par["K1"])-1)*exp((1/par["S"])*(par["P"]-temperatures)))^(-1/exp(par["K1"]))),
+                           ifelse(par["K2"]>3 & sign(par["P"]-temperatures)==sign(par["S"]), 
+                                  0.5*exp((temperatures-par["P"])/(par["S"]*exp(par["K2"]))), 
+                                  (1+(2^exp(par["K2"])-1)*exp((1/par["S"])*(par["P"]-temperatures)))^(-1/exp(par["K2"])))
+                           
+  )
+}
+if (equation=="Hulin") {
+  p <- (1+(2^exp(par["K1"]*x+par["K2"])-1)*exp((1/par["S"])*(par["P"]-x)))^(-1/exp(par["K1"]*x+par["K2"]))
+  prop.theorique <- (1+(2^exp(par["K1"]*temperatures+par["K2"])-1)*exp((1/par["S"])*(par["P"]-temperatures)))^(-1/exp(par["K1"]*temperatures+par["K2"]))
+}
 
  # prop.cumul <- NULL
  # for (i in 1:length(N)) prop.cumul <- c(prop.cumul, binom.test(males[i], N[i], prop.theorique[i])$p.value)
@@ -309,10 +454,6 @@ if (equation!="GSD") {
  # 
  # result$GOF <- combine.test(prop.cumul, method = c("z.transform"))
 
-	saturated <- 0
-  for (i in 1:length(N)) saturated <- saturated- dbinom(males[i], N[i], males[i]/N[i], log=TRUE)
-  LRT <- 2*(result$value-saturated)
-  result$GOF <- 1-pchisq(LRT, length(N)-length(par))
   
   par(new=TRUE)
 	if (males.freq) {   
@@ -323,6 +464,8 @@ if (equation!="GSD") {
   L1 <- modifyList(L1, list(ylim=c(0,1), axes=FALSE, xlab="", ylab="", type="l", main="", xlim=c(x1, x2)))
 
   a <- do.call(plot, L1)
+
+if (show.CI) {
   
   	pm <- NULL
 		pp <- NULL
@@ -330,6 +473,7 @@ if (equation!="GSD") {
   
 	if (equation=="logistic") {
     # ici je laisse S contrairement à predict.tsd
+    # 19/9/2014 Je ne comprends pas
 	  parameter.expand <- expand.grid(S=c(ifelse(sign(par["S"]-range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]-range.CI.qnorm*res["S"]), 
 	                                      ifelse(sign(par["S"]+range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]+range.CI.qnorm*res["S"])),
 	                                  P=c(par["P"]-range.CI.qnorm*res["P"], par["P"]+range.CI.qnorm*res["P"]), temperatures=x)
@@ -371,7 +515,52 @@ if (equation!="GSD") {
 
 	}
 	
+	if (equation=="Double-Richards") {
+	  parameter.expand <- expand.grid(S=c(ifelse(sign(par["S"]-range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]-range.CI.qnorm*res["S"]), 
+	                                      ifelse(sign(par["S"]+range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]+range.CI.qnorm*res["S"])),
+	                                  P=c(par["P"]-range.CI.qnorm*res["P"], par["P"]+range.CI.qnorm*res["P"]), 
+	                                  K1=c(ifelse(sign(par["K1"]-range.CI.qnorm*res["K1"])!=sign(par["K1"]), sign(par["K1"])*1e-5, par["K1"]-range.CI.qnorm*res["K1"]), 
+	                                      ifelse(sign(par["K1"]+range.CI.qnorm*res["K1"])!=sign(par["K1"]), sign(par["K1"])*1e-5, par["K1"]+range.CI.qnorm*res["K1"])), 
+	                                  K2=c(ifelse(sign(par["K2"]-range.CI.qnorm*res["K2"])!=sign(par["K2"]), sign(par["K2"])*1e-5, par["K2"]-range.CI.qnorm*res["K2"]), 
+	                                       ifelse(sign(par["K2"]+range.CI.qnorm*res["K2"])!=sign(par["K2"]), sign(par["K2"])*1e-5, par["K2"]+range.CI.qnorm*res["K2"])), 
+	                                  temperatures=x)
+	  y <- as.numeric(with(parameter.expand, ifelse(temperatures<P, 
+	                       ifelse(K1>3 & sign(P-temperatures)==sign(S), 
+	                              0.5*exp((temperatures-P)/(S*exp(K1))), 
+	                              (1+(2^exp(K1)-1)*exp((1/S)*(P-temperatures)))^(-1/exp(K1))),
+	                       ifelse(K2>3 & sign(P-temperatures)==sign(S), 
+	                              0.5*exp((temperatures-P)/(S*exp(K2))), 
+	                              (1+(2^exp(K2)-1)*exp((1/S)*(P-temperatures)))^(-1/exp(K2)))
+	                       
+	  )
+	  ))
+	  l <- split(y, parameter.expand$temperatures)
+	  
+	  pm <- unlist(lapply(l, min))
+	  pp <- unlist(lapply(l, max))
+	  
+	}
+	
   
+if (equation=="Hulin") {
+  parameter.expand <- expand.grid(S=c(ifelse(sign(par["S"]-range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]-range.CI.qnorm*res["S"]), 
+                                      ifelse(sign(par["S"]+range.CI.qnorm*res["S"])!=sign(par["S"]), sign(par["S"])*1e-5, par["S"]+range.CI.qnorm*res["S"])),
+                                  P=c(par["P"]-range.CI.qnorm*res["P"], par["P"]+range.CI.qnorm*res["P"]), 
+                                  K1=c(ifelse(sign(par["K1"]-range.CI.qnorm*res["K1"])!=sign(par["K1"]), sign(par["K1"])*1e-5, par["K1"]-range.CI.qnorm*res["K1"]), 
+                                      ifelse(sign(par["K1"]+range.CI.qnorm*res["K1"])!=sign(par["K1"]), sign(par["K1"])*1e-5, par["K1"]+range.CI.qnorm*res["K1"])), 
+                                  K2=c(ifelse(sign(par["K2"]-range.CI.qnorm*res["K2"])!=sign(par["K2"]), sign(par["K2"])*1e-5, par["K2"]-range.CI.qnorm*res["K2"]), 
+                                       ifelse(sign(par["K2"]+range.CI.qnorm*res["K2"])!=sign(par["K2"]), sign(par["K2"])*1e-5, par["K2"]+range.CI.qnorm*res["K2"])), 
+                                  temperatures=x)
+  y <- as.numeric(with(parameter.expand, 
+                       (1+(2^exp(K1*temperatures+K2)-1)*exp((1/S)*(P-temperatures)))^(-1/exp(K1*temperatures+K2)))
+  )
+  l <- split(y, parameter.expand$temperatures)
+  
+  pm <- unlist(lapply(l, min))
+  pp <- unlist(lapply(l, max))
+  
+}
+
   
 	if (!is.null(pp)) {
 	par(new=TRUE)
@@ -397,8 +586,15 @@ if (equation!="GSD") {
   a <- do.call(plot, L1) 
 
 	}  
-  
-  
+}
+# fin du plot==true
+}
+
+  saturated <- 0
+  for (i in 1:length(N)) saturated <- saturated- dbinom(males[i], N[i], males[i]/N[i], log=TRUE)
+  LRT <- 2*(result$value-saturated)
+  result$GOF <- 1-pchisq(LRT, length(N)-length(par))
+
   result$males <- males
   result$females <- females
   result$N <- N
@@ -407,7 +603,7 @@ if (equation!="GSD") {
   result$equation <- equation
   result$l <- l
   
-  if (print) print(paste("The goodness of fit test is", sprintf("%.3f",result$GOF)))
+  if (print) print(paste("The goodness of fit test is", sprintf("%.5f",result$GOF)))
 
 if (equation!="GSD" & print) {
   print(paste("The", lab.PT, "is", sprintf("%.3f",par["P"]), "SE", sprintf("%.3f",res["P"])))
@@ -418,6 +614,4 @@ if (equation!="GSD" & print) {
 
 	class(result) <- "tsd"
   return(invisible(result))
-  
-  
 }
